@@ -3,9 +3,10 @@ use std::fmt;
 use std::ops::Deref;
 
 use cosmwasm_std::{
-    Addr, Api, Attribute, BankMsg, Binary, BlockInfo, Coin, ContractInfo, ContractResult,
-    CustomQuery, Deps, DepsMut, Empty, Env, Event, MessageInfo, Order, Querier, QuerierWrapper,
-    Reply, ReplyOn, Response, Storage, SubMsg, SubMsgExecutionResponse, WasmMsg, WasmQuery,
+    to_binary, Addr, Api, Attribute, BankMsg, Binary, BlockInfo, Coin, ContractInfo,
+    ContractResult, CustomQuery, Deps, DepsMut, Env, Event, MessageInfo, Order, Querier,
+    QuerierWrapper, Reply, ReplyOn, Response, StdResult, Storage, SubMsg, SubMsgExecutionResponse,
+    TransactionInfo, WasmMsg, WasmQuery,
 };
 use cosmwasm_storage::{prefixed, prefixed_read, PrefixedStorage, ReadonlyPrefixedStorage};
 use prost::Message;
@@ -15,7 +16,7 @@ use serde::{Deserialize, Serialize};
 
 use cw_storage_plus::Map;
 
-use crate::app::{Router, RouterQuerier};
+use crate::app::{CosmosRouter, RouterQuerier};
 use crate::contracts::Contract;
 use crate::error::Error;
 use crate::executor::AppResponse;
@@ -30,6 +31,21 @@ const CONTRACTS: Map<&Addr, ContractData> = Map::new("contracts");
 pub const NAMESPACE_WASM: &[u8] = b"wasm";
 const CONTRACT_ATTR: &str = "_contract_addr";
 
+#[derive(Clone, std::fmt::Debug, PartialEq, JsonSchema)]
+pub struct WasmSudo {
+    pub contract_addr: Addr,
+    pub msg: Binary,
+}
+
+impl WasmSudo {
+    pub fn new<T: Serialize>(contract_addr: &Addr, msg: &T) -> StdResult<WasmSudo> {
+        Ok(WasmSudo {
+            contract_addr: contract_addr.clone(),
+            msg: to_binary(msg)?,
+        })
+    }
+}
+
 /// Contract Data includes information about contract, equivalent of `ContractInfo` in wasmd
 /// interface.
 #[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
@@ -46,10 +62,7 @@ pub struct ContractData {
     pub created: u64,
 }
 
-pub trait Wasm<ExecC = Empty, QueryC = Empty>
-where
-    ExecC: Clone + fmt::Debug + PartialEq + JsonSchema,
-{
+pub trait Wasm<ExecC, QueryC> {
     /// Handles all WasmQuery requests
     fn query(
         &self,
@@ -65,51 +78,45 @@ where
         &self,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         sender: Addr,
         msg: WasmMsg,
     ) -> AnyResult<AppResponse>;
 
-    // Add a new contract. Must be done on the base object, when no contracts running
-    fn store_code(&mut self, code: Box<dyn Contract<ExecC>>) -> usize;
-
-    // Helper for querying for specific contract data
-    fn contract_data(&self, storage: &dyn Storage, address: &Addr) -> AnyResult<ContractData>;
-
     /// Admin interface, cannot be called via CosmosMsg
     fn sudo(
         &self,
         api: &dyn Api,
         contract_addr: Addr,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
-        msg: Vec<u8>,
+        msg: Binary,
     ) -> AnyResult<AppResponse>;
 }
 
-pub struct WasmKeeper<ExecC = Empty, QueryC = Empty> {
+pub struct WasmKeeper<ExecC, QueryC> {
     /// code is in-memory lookup that stands in for wasm code
     /// this can only be edited on the WasmRouter, and just read in caches
     codes: HashMap<usize, Box<dyn Contract<ExecC>>>,
-    /// Just marker to make type elision fork when using it as `Wasm` trait
-    _q: std::marker::PhantomData<QueryC>,
+    /// Just markers to make type elision fork when using it as `Wasm` trait
+    _p: std::marker::PhantomData<QueryC>,
 }
 
 impl<ExecC, QueryC> Default for WasmKeeper<ExecC, QueryC> {
     fn default() -> Self {
         Self {
             codes: HashMap::default(),
-            _q: std::marker::PhantomData,
+            _p: std::marker::PhantomData,
         }
     }
 }
 
 impl<ExecC, QueryC> Wasm<ExecC, QueryC> for WasmKeeper<ExecC, QueryC>
 where
-    ExecC: Clone + fmt::Debug + PartialEq + JsonSchema + 'static,
-    QueryC: CustomQuery + DeserializeOwned,
+    ExecC: Clone + fmt::Debug + PartialEq + JsonSchema + DeserializeOwned + 'static,
+    QueryC: CustomQuery + DeserializeOwned + 'static,
 {
     fn query(
         &self,
@@ -136,7 +143,7 @@ where
         &self,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         sender: Addr,
         msg: WasmMsg,
@@ -148,37 +155,41 @@ where
         self.process_response(api, router, storage, block, resender, res, msgs)
     }
 
-    fn store_code(&mut self, code: Box<dyn Contract<ExecC>>) -> usize {
-        let idx = self.codes.len() + 1;
-        self.codes.insert(idx, code);
-        idx
-    }
-
-    fn contract_data(&self, storage: &dyn Storage, address: &Addr) -> AnyResult<ContractData> {
-        self.load_contract(storage, address)
-    }
-
     fn sudo(
         &self,
         api: &dyn Api,
         contract: Addr,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
-        msg: Vec<u8>,
+        msg: Binary,
     ) -> AnyResult<AppResponse> {
         let custom_event = Event::new("sudo").add_attribute(CONTRACT_ATTR, &contract);
 
-        let res = self.call_sudo(contract.clone(), api, storage, router, block, msg)?;
+        let res = self.call_sudo(contract.clone(), api, storage, router, block, msg.to_vec())?;
         let (res, msgs) = self.build_app_response(&contract, custom_event, res);
         self.process_response(api, router, storage, block, contract, res, msgs)
     }
 }
 
+impl<ExecC, QueryC> WasmKeeper<ExecC, QueryC> {
+    pub fn store_code(&mut self, code: Box<dyn Contract<ExecC>>) -> usize {
+        let idx = self.codes.len() + 1;
+        self.codes.insert(idx, code);
+        idx
+    }
+
+    pub fn load_contract(&self, storage: &dyn Storage, address: &Addr) -> AnyResult<ContractData> {
+        CONTRACTS
+            .load(&prefixed_read(storage, NAMESPACE_WASM), address)
+            .map_err(Into::into)
+    }
+}
+
 impl<ExecC, QueryC> WasmKeeper<ExecC, QueryC>
 where
-    ExecC: Clone + fmt::Debug + PartialEq + JsonSchema + 'static,
-    QueryC: CustomQuery + DeserializeOwned,
+    ExecC: Clone + fmt::Debug + PartialEq + JsonSchema + DeserializeOwned + 'static,
+    QueryC: CustomQuery + DeserializeOwned + 'static,
 {
     pub fn new() -> Self {
         Self::default()
@@ -209,21 +220,25 @@ where
         data.into()
     }
 
-    fn send<T: Into<Addr>>(
+    fn send<T>(
         &self,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         sender: T,
         recipient: String,
         amount: &[Coin],
-    ) -> AnyResult<AppResponse> {
+    ) -> AnyResult<AppResponse>
+    where
+        T: Into<Addr>,
+    {
         if !amount.is_empty() {
-            let msg = BankMsg::Send {
+            let msg: cosmwasm_std::CosmosMsg<ExecC> = BankMsg::Send {
                 to_address: recipient,
                 amount: amount.to_vec(),
-            };
+            }
+            .into();
             let res = router.execute(api, storage, block, sender.into(), msg.into())?;
             Ok(res)
         } else {
@@ -236,7 +251,7 @@ where
         &self,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         sender: Addr,
         wasm_msg: WasmMsg,
@@ -374,7 +389,7 @@ where
     fn execute_submsg(
         &self,
         api: &dyn Api,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         storage: &mut dyn Storage,
         block: &BlockInfo,
         contract: Addr,
@@ -386,7 +401,7 @@ where
 
         // execute in cache
         let res = transactional(storage, |write_cache, _| {
-            router.execute(api, write_cache, block, contract.clone(), msg)
+            router.execute(api, write_cache, block, contract.clone(), msg.into())
         });
 
         // call reply if meaningful
@@ -429,7 +444,7 @@ where
     fn _reply(
         &self,
         api: &dyn Api,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         storage: &mut dyn Storage,
         block: &BlockInfo,
         contract: Addr,
@@ -498,7 +513,7 @@ where
     fn process_response(
         &self,
         api: &dyn Api,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         storage: &mut dyn Storage,
         block: &BlockInfo,
         contract: Addr,
@@ -552,7 +567,7 @@ where
         api: &dyn Api,
         storage: &mut dyn Storage,
         address: Addr,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         info: MessageInfo,
         msg: Vec<u8>,
@@ -572,7 +587,7 @@ where
         address: Addr,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         info: MessageInfo,
         msg: Vec<u8>,
@@ -592,7 +607,7 @@ where
         address: Addr,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         reply: Reply,
     ) -> AnyResult<Response<ExecC>> {
@@ -611,7 +626,7 @@ where
         address: Addr,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         msg: Vec<u8>,
     ) -> AnyResult<Response<ExecC>> {
@@ -630,7 +645,7 @@ where
         address: Addr,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         msg: Vec<u8>,
     ) -> AnyResult<Response<ExecC>> {
@@ -650,6 +665,7 @@ where
             contract: ContractInfo {
                 address: address.into(),
             },
+            transaction: Some(TransactionInfo { index: 0 }),
         }
     }
 
@@ -685,13 +701,14 @@ where
         &self,
         api: &dyn Api,
         storage: &mut dyn Storage,
-        router: &Router<ExecC, QueryC>,
+        router: &dyn CosmosRouter<ExecC = ExecC, QueryC = QueryC>,
         block: &BlockInfo,
         address: Addr,
         action: F,
     ) -> AnyResult<T>
     where
         F: FnOnce(&Box<dyn Contract<ExecC>>, DepsMut, Env) -> AnyResult<T>,
+        ExecC: DeserializeOwned,
     {
         let contract = self.load_contract(storage, &address)?;
         let handler = self
@@ -717,12 +734,6 @@ where
         })
     }
 
-    pub fn load_contract(&self, storage: &dyn Storage, address: &Addr) -> AnyResult<ContractData> {
-        CONTRACTS
-            .load(&prefixed_read(storage, NAMESPACE_WASM), address)
-            .map_err(Into::into)
-    }
-
     pub fn save_contract(
         &self,
         storage: &mut dyn Storage,
@@ -854,25 +865,34 @@ pub fn parse_contract_addr(data: &Option<Binary>) -> AnyResult<Addr> {
 
 #[cfg(test)]
 mod test {
-    use crate::custom_handler::PanickingCustomHandler;
     use cosmwasm_std::testing::{mock_env, mock_info, MockApi, MockQuerier, MockStorage};
     use cosmwasm_std::{coin, from_slice, to_vec, BankMsg, Coin, CosmosMsg, Empty, StdError};
 
+    use crate::app::Router;
+    use crate::bank::BankKeeper;
+    use crate::module::FailingModule;
     use crate::test_helpers::contracts::{error, payout};
     use crate::transactions::StorageTransaction;
-    use crate::BankKeeper;
 
     use super::*;
+    use crate::staking::{FailingDistribution, FailingStaking};
 
-    fn mock_keeper() -> WasmKeeper<Empty> {
-        WasmKeeper::new()
-    }
+    /// Type alias for default build `Router` to make its reference in typical scenario
+    type BasicRouter<ExecC = Empty, QueryC = Empty> = Router<
+        BankKeeper,
+        FailingModule<ExecC, QueryC, Empty>,
+        WasmKeeper<ExecC, QueryC>,
+        FailingStaking,
+        FailingDistribution,
+    >;
 
-    fn mock_router() -> Router<Empty, Empty> {
+    fn mock_router() -> BasicRouter {
         Router {
-            wasm: Box::new(WasmKeeper::new()),
-            bank: Box::new(BankKeeper::new()),
-            custom: Box::new(PanickingCustomHandler),
+            wasm: WasmKeeper::new(),
+            bank: BankKeeper::new(),
+            custom: FailingModule::new(),
+            staking: FailingStaking::new(),
+            distribution: FailingDistribution::new(),
         }
     }
 
@@ -880,7 +900,7 @@ mod test {
     fn register_contract() {
         let api = MockApi::default();
         let mut wasm_storage = MockStorage::new();
-        let mut keeper = mock_keeper();
+        let mut keeper = WasmKeeper::new();
         let block = mock_env().block;
         let code_id = keeper.store_code(error::contract());
 
@@ -970,7 +990,7 @@ mod test {
     #[test]
     fn contract_send_coins() {
         let api = MockApi::default();
-        let mut keeper = mock_keeper();
+        let mut keeper = WasmKeeper::new();
         let block = mock_env().block;
         let code_id = keeper.store_code(payout::contract());
 
@@ -1045,7 +1065,7 @@ mod test {
     }
 
     fn assert_payout(
-        router: &WasmKeeper<Empty>,
+        router: &WasmKeeper<Empty, Empty>,
         storage: &mut dyn Storage,
         contract_addr: &Addr,
         payout: &Coin,
@@ -1081,7 +1101,7 @@ mod test {
     #[test]
     fn multi_level_wasm_cache() {
         let api = MockApi::default();
-        let mut keeper = mock_keeper();
+        let mut keeper = WasmKeeper::new();
         let block = mock_env().block;
         let code_id = keeper.store_code(payout::contract());
 
